@@ -1,24 +1,13 @@
-import numpy as np
 import math
-from passpredict.constants import (
-    R_EARTH,
-    R2_EARTH,
-    e_EARTH,
-    e2_EARTH,
-    MU,
-    J2,
-    J2000,
-    AU_M,
-    AU_KM,
-    ASEC360,
-    DAY_S,
-    ASEC2RAD,
-    DEG2RAD,
-    RAD2DEG,
-    tau,
-)
 from math import pi
-from passpredict.topocentric import site_ECEF, site_declination_and_K
+
+import numpy as np
+
+from .constants import tau, ASEC2RAD, DEG2RAD
+from .topocentric import site_ECEF, site_declination_and_K
+from .precession import fk5_precession
+from .nutation import fk5_nutation
+from .timefn import jd2jc
 
 # _arrays = np.load('nutation.npz')
 # lunisolar_longitude_coefficients = _arrays['lunisolar_longitude_coefficients']
@@ -28,7 +17,7 @@ from passpredict.topocentric import site_ECEF, site_declination_and_K
 # nutation_coefficients_longitude = _arrays['nutation_coefficients_longitude']
 # nutation_coefficients_obliquity = _arrays['nutation_coefficients_obliquity']
 
-
+# @profile
 def ecef2sez(r, phi, lmda):
     """
     Rotate r vector from ECEF frame to SEZ frame
@@ -151,12 +140,26 @@ def equinox1982(dPsi1980, eps1980, omega_moon):
     References:
         Vallado, p.224
     """
-    eq = dPsi1980 * np.cos(eps1980)
+    eq = equinox1982_geometric_terms(dPsi1980, eps1980)
     eq += 0.00264 * ASEC2RAD * np.sin(omega_moon)
     eq += 0.000063 * np.sin(2 * omega_moon)
     return eq
 
 
+def equinox1982_geometric_terms(dPsi1980, eps1980):
+    """Return the geometric terms of equation of equinoxes for sidereal time
+    for IAU-76/FK5 reductions
+
+    Args:
+        dPsi1980 : float  (radians)
+        eps1920 : float   (radians)
+
+    References:
+        Vallado, p.224
+    """
+    return dPsi1980 * np.cos(eps1980)
+
+
 def theta_GAST1982(Eq, gmst):
     """Return the Greenwich apparent sidereal time
 
@@ -172,134 +175,20 @@ def theta_GAST1982(Eq, gmst):
     return Eq + gmst
 
 
-def nutation_coeff():
-    i = np.array(
-        [1, 9, 31, 2, 10, 32, 11, 33, 34, 12, 35, 13, 36, 38, 37], dtype=np.int
-    )
-    A = np.array(
-        [
-            -171996,
-            -13187,
-            -2274,
-            2062,
-            1426,
-            712,
-            -517,
-            -386,
-            -301,
-            217,
-            -158,
-            129,
-            123,
-            63,
-            63,
-        ],
-        dtype=np.float64,
-    )
-    B = np.array(
-        [
-            -174.2,
-            -1.6,
-            -0.2,
-            0.2,
-            -3.4,
-            0.1,
-            1.2,
-            -0.4,
-            0.0,
-            -0.5,
-            0.0,
-            0.1,
-            0.0,
-            0.1,
-            0.0,
-        ]
-    )
-    C = np.array(
-        [92025, 5736, 977, -895, 54, -7, 224, 200, 129, -95, -1, -70, -53, -33, -2]
-    )
-    D = np.array(
-        [8.9, -3.1, -0.5, 0.5, -0.1, 0.0, -0.6, 0.0, -0.1, 0.3, 0.0, 0.0, 0.0, 0.0, 0.0]
-    )
-    an = np.array(
-        [
-            [0, 0, 0, 0, 1],
-            [0, 0, 2, -2, 2],
-            [0, 0, 2, 0, 2],
-            [0, 0, 0, 0, 2],
-            [0, 1, 0, 0, 0],
-            [1, 0, 0, 0, 0],
-            [0, 1, 2, -2, 2],
-            [0, 0, 2, 0, 1],
-        ]
-    )
 
 
-def fk5_nutation(tt):
-    """
-    Ref: Table D-6, p. 1043
-    """
-    api = an1 * M_moon + an2 * M_sun + an3 * Um_moon + an4 * D_sun + an5 * Omega_moon
-    psi_tmp = np.sum(A + B * tt) * np.sin(api)
-    eps_tmp = np.sum(C + D * tt) * np.cos(api)
 
-    return dPsi1980, dEps1980
+# def fk5_nutation(tt):
+#     """
+#     Ref: Table D-6, p. 1043
+#     """
+#     api = an1 * M_moon + an2 * M_sun + an3 * Um_moon + an4 * D_sun + an5 * Omega_moon
+#     psi_tmp = np.sum(A + B * tt) * np.sin(api)
+#     eps_tmp = np.sum(C + D * tt) * np.cos(api)
+
+#     return dPsi1980, dEps1980
 
 
-def fk5_precession(Td):
-    """Calculate precession angle Theta
-
-    Assume JD2000 epoch, T0 = 0
-
-    Args
-        Td: terrestial time
-
-    References:
-        Vallado, p. 227, Eq. 3-87
-    """
-    # Td = (jdt - J2000)/36525
-    zeta = (2306.2181 + (0.30188 + 0.017998 * Td) * Td) * Td
-    theta = (2004.3109 + (-0.42665 - 0.041833 * Td) * Td) * Td
-    z = (2306.2181 + (1.09468 + 0.018203 * Td) * Td) * Td
-    # Convert to radians
-    zeta *= ASEC2RAD
-    theta *= ASEC2RAD
-    z *= ASEC2RAD
-
-    return zeta, theta, z
-
-
-def precess_rotation(r, zeta, theta, z):
-    """Perform precession rotations on position vector r
-    From IAU 80 Precession Theory. Rotation from MOD to GCRF.
-
-    Args:
-        r : (3, n), n is the number of observations
-        zeta, theta, z (n): precession angles in radians
-
-    References:
-        Vallado, p. 228, Eq. 3-89
-    """
-    coszeta = np.cos(zeta)
-    sinzeta = np.sin(zeta)
-    costheta = np.cos(theta)
-    sintheta = np.sin(theta)
-    cosz = np.cos(z)
-    sinz = np.sin(z)
-
-    rGCRF = np.empty(r.shape, dtype=np.float)
-    rGCRF[0] = (
-        (costheta * cosz * coszeta - sinz * sinzeta) * r[0]
-        + (sinz * costheta * coszeta + sinzeta * cosz) * r[1]
-        + sintheta * coszeta * r[2]
-    )
-    rGCRF[1] = (
-        (-sinzeta * costheta * cosz - sinz * coszeta) * r[0]
-        + (-sinz * sinzeta * costheta + cosz * coszeta) * r[1]
-        - sintheta * sinzeta * r[2]
-    )
-    rGCRF[2] = -sintheta * cosz * r[0] - sintheta * sinz * r[1] + costheta * r[2]
-    return rGCRF
 
 
 def teme2ecef(r, jdt, xp=0.0, yp=0.0):
@@ -591,6 +480,46 @@ def ecef2eci(r, jdt):
     return rECEF
 
 
+def teme2eci(rTEME, jdt):
+    """Convert TEME vectors to GCRS Earth centered interial coordinates
+
+    TEME -> TOD -> MOD -> J2000 (ECI)
+
+    Reference:
+        teme2eci.m Vallado software
+
+    """
+    tt = jd2jc(jdt)  # convert julian date to julian century
+    prec = fk5_precession(tt)
+    nut = fk5_nutation(tt)
+    eq_equinox_star = equinox1982_geometric_terms(nut.dpsi, nut.meaneps)
+    E = rot3(-eq_equinox_star)
+    print(f'prec = {prec}')
+    print(f'nut  = {nut}')
+    print(f'E    = {E}')
+    # Create rotation matrix  [M] = [P][N][E]
+    M = np.dot(np.dot(prec.mtx, nut.mtx), E)
+    rJ2000 = mxv(M, rTEME)
+    return rJ2000
+
+
+def mxv(mtx, vec):
+    """
+    Multiply a matrix by (m) vectors
+
+    Args:
+        mtx: float (n, n)
+        vec: float (n, m)
+
+    Returns:
+        float (n, m)
+
+    Reference:
+        skyfield/functions.py, line 23
+    """
+    return np.einsum('ij...,j...->i...', mtx, vec)
+
+
 def rot1(a):
     """Compute Euler angle rotation matrix, first angle
 
@@ -638,10 +567,10 @@ def rot3(a):
     )
     return mtx
 
-
+# @profile
 def site_sat_rotations(lat, lon, h, rsatECEF):
     rsiteECEF = site_ECEF(lat, lon, h)
-    rho = rsatECEF - np.atleast_2d(rsiteECEF).T
+    rho = rsatECEF - np.array([[rsiteECEF[0]],[rsiteECEF[1]],[rsiteECEF[2]]], dtype=np.float64)
     rSEZ = ecef2sez(rho, lat, lon)
     return rSEZ
 
